#include "stdafx.h"
#include "WaveEd.h"
#include "ChildView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CChildView::CChildView()
{
	WorkspaceView = true;
	DrawWave = true;
	SelectionLength = 0;
	SelectionStart = 0;
	DisplayStart = 0;
	DisplayLength = 0;
}

CChildView::~CChildView()
{
	DeleteObject(hBgBrush);
}


BEGIN_MESSAGE_MAP(CChildView,CWnd )
	//{{AFX_MSG_MAP(CChildView)
	ON_WM_PAINT()
	ON_COMMAND(ID_VIEW_WORKSPACE, OnViewWorkspace)
	ON_WM_LBUTTONDOWN()
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONUP()
	ON_UPDATE_COMMAND_UI(ID_EDIT_CUT, OnUpdateEditCut)
	ON_UPDATE_COMMAND_UI(ID_EDIT_COPY, OnUpdateEditCopy)
	ON_UPDATE_COMMAND_UI(ID_EDIT_DELETE, OnUpdateEditDelete)
	ON_UPDATE_COMMAND_UI(ID_EDIT_PASTE, OnUpdateEditPaste)
	ON_UPDATE_COMMAND_UI(ID_BAR_ZOOMIN, OnUpdateBarZoomin)
	ON_UPDATE_COMMAND_UI(ID_BAR_ZOOMOUT, OnUpdateBarZoomout)
	ON_UPDATE_COMMAND_UI(ID_BAR_SHOWALL, OnUpdateBarShowall)
	ON_COMMAND(ID_BAR_ZOOMIN, OnBarZoomin)
	ON_COMMAND(ID_BAR_SHOWALL, OnBarShowall)
	ON_COMMAND(ID_BAR_ZOOMOUT, OnBarZoomout)
	ON_COMMAND(ID_EDIT_CUT, OnEditCut)
	ON_COMMAND(ID_EDIT_COPY, OnEditCopy)
	ON_COMMAND(ID_EDIT_DELETE, OnEditDelete)
	ON_COMMAND(ID_EDIT_PASTE, OnEditPaste)
	ON_UPDATE_COMMAND_UI(ID_VIEW_WORKSPACE, OnUpdateViewWorkspace)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CChildView message handlers

BOOL CChildView::PreCreateWindow(CREATESTRUCT& cs) 
{
	LOGBRUSH LogB;
	LogB.lbColor = 0x00000000;
	LogB.lbStyle = BS_SOLID;

	hBgBrush = CreateBrushIndirect(&LogB);
	
	cs.dwExStyle |= WS_EX_CLIENTEDGE;
	cs.style &= ~WS_BORDER;
	cs.lpszClass = AfxRegisterWndClass(CS_HREDRAW|CS_VREDRAW|CS_DBLCLKS, 
	::LoadCursor(NULL, IDC_ARROW), hBgBrush, NULL);

	return CWnd::PreCreateWindow(cs);
}

void CChildView::OnPaint() 
{
	CPaintDC dc(this);
	CRect rect;
	double OffsetStep = 0;
	Peaks PeaksInRange;
	
	static DWORD SelectionX1 = 0, SelectionX2 = 0;
	
	GetClientRect(&rect);
	dc.SetBkColor(0x000000);
	dc.SetTextColor(0xffffff);

	if (DrawWave)
	{

		// Stereo channels separator line
		dc.SelectObject(&pen_1);
		dc.MoveTo(0,rect.Height() >> 1);
		dc.LineTo(rect.Width(), rect.Height() >> 1);
		if (!this->pEngine->IsEmpty())
		{
			if (!(WorkspaceView && !pEngine->GetWorkspaceSize()))
			{
				OffsetStep = (double) DisplayLength / rect.Width();
	#pragma warning (push, 3)
				for (DWORD i = 0; i < rect.Width(); i++)
	#pragma warning (pop)
				{
					DWORD Offset = DisplayStart + (i * OffsetStep);

					if (WorkspaceView) PeaksInRange = GetPeaksInRange(Offset, OffsetStep, CV_WORKSPACE);
					else PeaksInRange = GetPeaksInRange(Offset, OffsetStep, CV_SINGLEWAVE);

					//przyspieszacze;
					int rectHeightDiv4 = rect.Height()/4;
					int rectHeightDiv4Mul3 = rectHeightDiv4*3;

					int lLow = (rectHeightDiv4 * PeaksInRange.L_lo)/32768;
					int lHi =  (rectHeightDiv4 * PeaksInRange.L_hi)/32768;
					int rLow = (rectHeightDiv4 * PeaksInRange.R_lo)/32768;
					int rHi =  (rectHeightDiv4 * PeaksInRange.R_hi)/32768;
					
					if (lLow == lHi) dc.SetPixel(i, rectHeightDiv4, 0x00ff00);
					else
					{
						dc.MoveTo(i, rectHeightDiv4 - lLow);
						dc.LineTo(i, rectHeightDiv4 - lHi);
					}

					if (rLow == rHi) dc.SetPixel(i, rectHeightDiv4Mul3, 0x00ff00);
					else
					{
						dc.MoveTo(i, rectHeightDiv4Mul3 - rLow);
						dc.LineTo(i, rectHeightDiv4Mul3 - rHi);
					}
				}
			}
		}
		else
		{
			dc.TextOut((rect.Width()>>1) - 48,(rect.Height()>>1) - 8,"No Wave Data");
		}
	}
	
		if (SelectionLength && !MoveInactive)
		{
			//dc.SelectObject(&pen_2);
			dc.SetROP2(R2_NOT);
			if (!DrawWave) dc.Rectangle(SelectionX1,0,SelectionX2,rect.Height());
			double invRatio = (double)rect.Width()/DisplayLength;
			SelectionX1=(SelectionStart-DisplayStart)*invRatio;
			SelectionX2=(SelectionStart+SelectionLength-DisplayStart)*invRatio;
			dc.Rectangle(SelectionX1,0,SelectionX2,rect.Height());
		}
		else
		{
			SelectionX1 = 0;
			SelectionX2 = 0;
		}
	

	DrawWave = true;
}


BOOL CChildView::OnCmdMsg(UINT nID, int nCode, void* pExtra, AFX_CMDHANDLERINFO* pHandlerInfo) 
{
	if (pHandlerInfo == NULL)
	{
		if ((nID >= ID_WAVESELECT_000) && (nID <= ID_WAVESELECT_000 + 255))
		{
			if (nCode == CN_COMMAND)
			{
				Wave = nID - ID_WAVESELECT_000;
				WorkspaceView = false;
				DisplayStart = 0;
				DisplayLength = pEngine->GetWaveSize(Wave);
				SelectionStart =0;
				SelectionLength = 0;
				Invalidate(true);
				return true;
			}
			else if (nCode == CN_UPDATE_COMMAND_UI)
			{
				((CCmdUI*)pExtra)->Enable(TRUE);
				((CCmdUI*)pExtra)->SetCheck((Wave == nID - ID_WAVESELECT_000) && !WorkspaceView);

				return true;
			}
		}
	}
	return CWnd ::OnCmdMsg(nID, nCode, pExtra, pHandlerInfo);
}

void CChildView::OnViewWorkspace() 
{
	WorkspaceView = true;
	DisplayStart = 0;
	DisplayLength = pEngine->GetWorkspaceSize();
	SelectionStart = 0;
	SelectionLength = 0;

	Invalidate(true);
}

void CChildView::AttachEngine(WaveEngine *_pEngine, WavePlayer* _pPlayer)
{
	this->pEngine = _pEngine;
	this->pPlayer = _pPlayer;
}

void CChildView::ResetView(int waveNumber)
{
	DrawWave = true;
	Wave = waveNumber;
	//Przechodzimy do wyswietlania pojedynczego wave'a.... niekoniecznie
	if (waveNumber == -1)
	{
		WorkspaceView = true;
		DisplayLength = pEngine->GetWorkspaceSize();
	}
	else
	{
		WorkspaceView = false;
		DisplayLength = pEngine->GetWaveSize(waveNumber);
	}

	DisplayStart = 0;
	SelectionLength = 0;
	SelectionStart = 0;
	Invalidate(true);
	UpdateWindow();
}

BOOL CChildView::Create(LPCTSTR lpszClassName, LPCTSTR lpszWindowName, DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID, CCreateContext* pContext) 
{
	pen_1.CreatePen(PS_SOLID,0,0x00FF00);
	pen_2.CreatePen(PS_SOLID,0,0xFFFFFF);
	
	return CWnd::Create(lpszClassName, lpszWindowName, dwStyle, rect, pParentWnd, nID, pContext);
}

Peaks CChildView::GetPeaksInRange(DWORD start, double step, bool mode)
{
	WORD iStep = (long)(step <1 ? 1 : step);
	DWORD* buff = new DWORD[iStep];
	DWORD* sample;
	DWORD filled;
	Peaks tmpPeaks = {0, 0, 0, 0};

	if (mode == CV_SINGLEWAVE)
	{pEngine->GetWaveDataBlock(Wave, (char*)buff, iStep, &filled, start);}
	else pEngine->GetDataBlock((char*)buff, iStep, &filled, start);
	
	for (DWORD i = 0; i < filled; i++)
	{
		sample = buff + i;
		if ( tmpPeaks.L_lo > *(SHORT*)sample ) tmpPeaks.L_lo = *(SHORT*)sample;
		if ( tmpPeaks.L_hi < *(SHORT*)sample ) tmpPeaks.L_hi = *(SHORT*)sample;

		if ( tmpPeaks.R_lo > *((SHORT*)sample+1) ) tmpPeaks.R_lo = *((SHORT*)sample+1);
		if ( tmpPeaks.R_hi < *((SHORT*)sample+1) ) tmpPeaks.R_hi = *((SHORT*)sample+1);
	}

	delete [] buff;

	return tmpPeaks;
}

void CChildView::OnLButtonDown(UINT nFlags, CPoint point)
{
	CRect rect;
	
	GetClientRect(&rect);
	SetCapture();
	float Ratio = (float)DisplayLength/rect.Width();
	
	if (!pEngine->IsEmpty())
	{
		if (!(WorkspaceView && !pEngine->GetWorkspaceSize()))
		{
			SelectionLength = 0;
			SelectionStart=DisplayStart+(point.x*Ratio);
			WhereLastClicked = point.x;
			DrawWave = true;
			Invalidate(true);
			UpdateWindow();
		}
	}

	CWnd::OnLButtonDown(nFlags, point);
}

void CChildView::OnMouseMove(UINT nFlags, CPoint point)
{
	CRect rect;
	int x = point.x;
	GetClientRect(&rect);
	double Ratio = (double)DisplayLength/rect.Width();

	if ((!pEngine->IsEmpty()) && (nFlags == MK_LBUTTON) && !MoveInactive)
	{
		if (!(WorkspaceView && !pEngine->GetWorkspaceSize()))
		{
			if (x >= (long) WhereLastClicked)
			{
				if (x > rect.Width())	{ x = rect.Width(); }
				int x1 = x*Ratio;
				SelectionLength = (x*Ratio + DisplayStart - SelectionStart);
			}
						
			else
			{
				if (x < 0) { x = 0; }
				SelectionStart = (DWORD) ( x*Ratio + DisplayStart);
				SelectionLength = (DWORD) (WhereLastClicked*Ratio + DisplayStart - SelectionStart);
			}

			DrawWave = false;
			Invalidate(false);
		}
	}
	MoveInactive = false;

	CWnd::OnMouseMove(nFlags, point);
}

void CChildView::OnLButtonUp(UINT nFlags, CPoint point)
{
	ReleaseCapture();
	CWnd::OnLButtonUp(nFlags, point);
}



/////////////Odswiezanie stanu menusow.///////////////
void CChildView::OnUpdateEditCut(CCmdUI* pCmdUI) 
{
	//Czyli cut dziaa tylko w trybie workspace :(
	//Brak czasu...
	pCmdUI->Enable(WorkspaceView && !pEngine->IsEmpty() && SelectionLength && !pPlayer->_isPlaying());
}

void CChildView::OnUpdateEditCopy(CCmdUI* pCmdUI) 
{
	//Copy dzaa zawsze, chyba ze nic nie jest wczytane.
	pCmdUI->Enable(!pEngine->IsEmpty() && SelectionLength);
}

void CChildView::OnUpdateEditDelete(CCmdUI* pCmdUI) 
{
	//Podobnie jak cut...
	pCmdUI->Enable(WorkspaceView && !pEngine->IsEmpty() && SelectionLength);
}

void CChildView::OnUpdateEditPaste(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(WorkspaceView && !pEngine->IsClipboardEmpty() && !pPlayer->_isPlaying());	
}

void CChildView::OnUpdateBarZoomin(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(!pEngine->IsEmpty() && SelectionLength);
	
}

void CChildView::OnUpdateBarZoomout(CCmdUI* pCmdUI) 
{
	if (!pEngine->IsEmpty())
	{
		if ((DisplayStart !=0) || ((DisplayLength + DisplayStart) != GetViewedDataSize()))
		{
			pCmdUI->Enable(true);
			return;
		}
	}

	pCmdUI->Enable(false);
}

void CChildView::OnUpdateBarShowall(CCmdUI* pCmdUI) 
{
	if (!pEngine->IsEmpty())
	{
		if ((DisplayStart !=0) || ((DisplayLength + DisplayStart) != GetViewedDataSize()))
		{
			pCmdUI->Enable(true);
			return;
		}
	}

	pCmdUI->Enable(false);
}

void CChildView::OnBarZoomin() 
{
	CRect rect;
	GetClientRect(&rect);
	float Ratio = (float)SelectionLength/rect.Width();

	if (Ratio > 0.5f)
	{
		DisplayStart = SelectionStart;
		DisplayLength = SelectionLength;
		Invalidate(true);
	}
}

void CChildView::OnBarShowall() 
{
	DisplayStart = 0;
	DisplayLength = GetViewedDataSize();
	Invalidate(true);
}

void CChildView::OnBarZoomout() 
{
	int tDStart;

	tDStart = DisplayStart - DisplayLength*0.25f;
	if (tDStart < 0) DisplayStart = 0;
	else DisplayStart = tDStart;

	DisplayLength += DisplayLength*0.5f;

	if ( (DisplayStart + DisplayLength) > GetViewedDataSize())
	{
		DisplayLength = GetViewedDataSize() - DisplayStart;
	}

	Invalidate(true);
}

DWORD CChildView::GetViewedDataSize(void)
{
	if (WorkspaceView) return pEngine->GetWorkspaceSize();
	else return pEngine->GetWaveSize(Wave);
}

void CChildView::OnEditCut() 
{
	pEngine->Cut(SelectionStart, SelectionStart+SelectionLength);
	if ( (DisplayStart + DisplayLength) > GetViewedDataSize())
	{
		DisplayLength = GetViewedDataSize() - DisplayStart;
	}
	SelectionLength = 0;
	Invalidate(true);
}

void CChildView::OnEditCopy() 
{
	if (WorkspaceView)
	{
		pEngine->Copy(SelectionStart, SelectionStart+SelectionLength);
	}
	else pEngine->SingleCopy(SelectionStart, SelectionStart+SelectionLength, Wave);
}

void CChildView::OnEditDelete() 
{
	pEngine->Delete(SelectionStart, SelectionStart+SelectionLength);
	if ( (DisplayStart + DisplayLength) > GetViewedDataSize())
	{
		DisplayLength = GetViewedDataSize() - DisplayStart;
	}
	SelectionLength = 0;
	Invalidate(true);
}

void CChildView::OnEditPaste() 
{
	if (SelectionLength)
	{
		pEngine->Delete(SelectionStart, SelectionStart+SelectionLength);
		SelectionLength = 0;
	}
	pEngine->Paste(SelectionStart);
	DisplayLength = GetViewedDataSize() - DisplayStart;
	Invalidate(true);
}

DWORD CChildView::GetSelectionStart(void)
{
	return SelectionStart;
}

DWORD CChildView::GetSelectionLength(void)
{
	return SelectionLength;
}

DWORD CChildView::GetDispStart(void)
{
	return DisplayStart;
}

DWORD CChildView::GetDispLength(void)
{
	return DisplayLength;
}

void CChildView::OnUpdateViewWorkspace(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(WorkspaceView);
}

bool CChildView::isWorkspaceDisplayed(void)
{
	return WorkspaceView;
}

void CChildView::DoNotMove(void)
{
	MoveInactive = true;
}